#!/usr/bin/python3
# -*- coding: utf-8 -*-

from pokemon import Pokemon
from combat import Combat
from random import randint

# ======================== VARIABLES GLOBALES =====================================
NB_VICTOIRES = 2  # Nombre de victoires à atteindre pour GAGNER
NB_DEFAITES = 2  # Nombre de défaites à atteindre pour GAME OVER
# =================================================================================

class JeuError(Exception):

    def __init__(self, msg):
        self.message = msg


# ===========================================================================================================
#                                               FONCTION UTILES
# ===========================================================================================================
def get_donnees_pokemons():
    ''' Lit le fichier "pokemons.txt" et récupère toutes les données sous forme d'une liste de dictionnaires.
    Chaque dictionnaire correspond aux données sur un pokémon. '''
    
    tableau_donnees = []
    with open('pokemons.txt') as f:
        i = 0
        j = 0
        line = f.readline()
        while line:
            if i == 0:
                line = line[0:-1]  # Enlever le '\n' de la fin
                tab = line.split('\t')
                tableau_donnees.append({})
                tableau_donnees[j]["nom"] = tab[0]
                tableau_donnees[j]["types"] = tab[1].split(',')
                tableau_donnees[j]["pts_vies"] = int(tab[2])
                tableau_donnees[j]["attaque"] = int(tab[3])
                tableau_donnees[j]["defense"] = int(tab[4])

                i += 1
            elif i == 1:
                tab = line.split('\t')
                tableau_donnees[j]["l_attaques"] = []
                for k in range(len(tab)):
                    t = tab[k].split(',')
                    dic = {"nom_attaque": t[0]}
                    if len(t) == 2:
                        t[1] = int(t[1])
                        dic["degats_attaque"] = t[1]
                    else:
                        t[2] = int(t[2])
                        dic["nom_effet"] = t[1]
                        dic["valeur_effet"] = t[2]
                    tableau_donnees[j]["l_attaques"].append(dic)
                i += 1
            else:
                i = 0
                j += 1

            line = f.readline()

        return tableau_donnees


def creer_pokemons(donnees):
    ''' Renvoie une liste d'objets POKEMON créées à partir des données donnes passées en argument. '''
    
    listep = []
    for el in donnees:
        listep.append(Pokemon(el["nom"], el["types"], el["pts_vies"], el["attaque"],
                              el["defense"], el["l_attaques"]))
    return listep


# ===========================================================================================================
#                                               CLASSE JEU
# ===========================================================================================================

class Jeu:
    ''' Classe Pokemon.
    Ses attributs sont :
    - _pokemons : Liste de tous les pokémons créés à partir du fichier pokemons.txt
    - _nb_victoires : Nombre de victoires remportées par le joueur
    - _nb_defaites : Nombre de défaites
    - _nb_combats_joues : Nombre total de combats joués '''

    def __init__(self):
        donnees = get_donnees_pokemons()
        self._pokemons = creer_pokemons(donnees)
        self._nb_victoires = 0
        self._nb_defaites = 0
        self._nb_combats_joues = 0

        jeu_fini = False
        while not jeu_fini:  # Tant qu'on a pas atteint le nombre de victoires ou de défaites spécifié
            if self.get_nb_victoires() == NB_VICTOIRES:
                print("BRAVO ! VOUS AVEZ GAGNE !")
                jeu_fini = True  # Fin du jeu
            elif self.get_nb_defaites() == NB_DEFAITES:
                print("GAME OVER ! Vous avez perdu trop de combats...")
                jeu_fini = True  # Fin du jeu
            else:
                self.deroulement_combat()

    # DEFINITION DES GETTERS

    def get_nb_victoires(self):
        ''' A COMPLETER - Renvoie le nombre de victoires remportées par le joueur. '''
        ..................

    def get_nb_defaites(self):
        ''' A COMPLETER - Renvoie le nombre de défaites remportées par le joueur. '''
        ..................

    def get_nb_combats_joues(self):
        ''' A COMPLETER - Renvoie le nombre total de combats joués. '''
        ..................

    # -----

    # DEFINITION DES SETTERS

    def ajouter_victoire(self):
        ''' A COMPLETER - Ajoute 1 victoire au nombre de victoires. '''
        ..................

    def ajouter_defaite(self):
        ''' A COMPLETER - Ajoute 1 victoire au nombre de défaites. '''
        ..................

    def ajouter_combat_joue(self):
        ''' A COMPLETER - Ajoute 1 victoire au nombre de combats joués. '''
        ..................

    # -----

    def pokemon_aleatoire(self, pokemon_deja_choisi):
        ''' A COMPLETER - Renvoie un pokémon choisi aléatoirement dans la liste _pokemons, parmi ceux encore en vie.
        De plus, on s'assure de ne pas renvoier un pokémon déjà choisi par le joueur.
        :param pokemon_deja_choisi: (str) Nom du pokémon choisi par le joueur '''

        tab_vivants = [pkm for pkm in self._pokemons if not pkm.est_mort() and not pkm.get_nom() == pokemon_deja_choisi]
        if tab_vivants:
            return ...............  # Renvoie un pokémon aléatoire de tab_vivants (utiliser randint)
        else:
            JeuError("Erreur : Il n'y a plus assez de pokémons disponibles pour jouer.")

    def choix_pokemon(self):
        ''' A COMPLETER - Fonction qui :
        - Affiche la liste des pokémons encore vivants
        - Demande à l'utilisateur de choisir le pokémon avec lequel il souhaite combattre
        Ce pokémon est ensuite renvoyé. '''

        print("Liste des pokémons encore vivants : \n")
        trouve = False
        for i in range(len(self._pokemons)):  # On parcourt la liste des pokémons _pokemons
            if not ................. :  # Si le pokémon n'EST PAS MORT
                trouve = True
                print(f"{i + 1} : {self._pokemons[i].get_nom()}")

        if not trouve:
            JeuError("Erreur : Il n'y a plus assez de pokémons disponibles pour jouer.")
        else:
            choix = input("Entrez le numéro du pokemon que vous souhaitez utiliser : ")
            return ...........  # Renvoie le pokémon de la liste _pokemons choisi par l'utilisateur

    def soigner_pokemons(self):
        ''' Fonction permettant de restaurer les caractéristiques (pts de vie, état, attaque, défense...)
        de tous les pokémons encore en vie à leur état initial, en allant rechercher les éléments dans le fichier 'pokemons.txt'. '''

        with open('pokemons.txt') as f:
            i = 0
            j = 0
            line = f.readline()
            while line:
                if i == 0:
                    if not self._pokemons[j].est_mort():
                        line = line[0:-1]  # Enlever le '\n' de la fin
                        tab = line.split('\t')
                        pts_vies = int(tab[2])
                        attaque = int(tab[3])
                        defense = int(tab[4])
                        self._pokemons[j].set_vies(pts_vies)
                        self._pokemons[j].set_attaque(attaque)
                        self._pokemons[j].set_defense(defense)
                        self._pokemons[j].set_etat({"nom_etat": "normal"})

                    i += 1
                elif i == 1:
                    i += 1
                else:
                    i = 0
                    j += 1

                line = f.readline()


    def deroulement_combat(self):
        self.soigner_pokemons()
        pokemon_joueur = self.choix_pokemon()
        pokemon_adversaire = self.pokemon_aleatoire(pokemon_joueur.get_nom())

        print(f"LANCEMENT DU COMBAT NUMERO {self.get_nb_combats_joues() + 1}")
        combat = Combat(pokemon_joueur, pokemon_adversaire)
        resultat_combat = combat.jouer()
        if resultat_combat:  # En cas de victoire
            self.ajouter_victoire()
        else:
            self.ajouter_defaite()
        self.ajouter_combat_joue()
